from setuptools import setup, find_packages


setup (
  name='pygments-enaml',
  version='0.1.0',
  packages=find_packages(),
  install_requires=['pygments'],
  entry_points=
  """
 [pygments.lexers]
 enaml = pygments_enaml.lexer:EnamlLexer
 """,
)



#------------------------------------------------------------------------------
# Copyright (c) 2013-2017, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
#------------------------------------------------------------------------------
"""Pytest fixtures.

"""
import os
from traceback import format_exc

# Make sure enaml already imported qt to avoid issues with pytest
try:
    from enaml.qt import QT_API, PYQT5_API, PYSIDE2_API
    if QT_API in PYQT5_API:
        os.environ.setdefault('PYTEST_QT_API', 'pyqt5')
    elif QT_API in PYSIDE2_API:
        os.environ.setdefault('PYTEST_QT_API', 'pyside2')

    pytest_plugins = (str('pytest-qt'),)

except Exception:
    pass

import pytest
from utils import close_all_windows, close_all_popups

#: Global variable linked to the --enaml-sleep cmd line option.
DIALOG_SLEEP = 0


def pytest_addoption(parser):
    """Add command line options.

    """
    parser.addoption("--enaml-sleep", action='store', type=float,
                     help="Time to sleep after handling a ui event")


def pytest_configure(config):
    """Turn the --enaml-sleep command line into a global variable.

    """
    s = config.getoption('--enaml-sleep')
    if s is not None:
        global DIALOG_SLEEP
        DIALOG_SLEEP = s


@pytest.fixture
def enaml_sleep():
    """Return the time to sleep in s as set by the --enaml-sleep option.

    """
    return DIALOG_SLEEP


@pytest.yield_fixture(scope='session')
def qt_app():
    """Make sure a QtApplication is active.

    """
    try:
        from enaml.qt.qt_application import QtApplication
    except Exception:
        pytest.skip('No Qt binding found: %s' % format_exc())

    app = QtApplication.instance()
    if app is None:
        app = QtApplication()
        yield app
        app.stop()
    else:
        yield app


@pytest.yield_fixture
def enaml_qtbot(qt_app, qtbot):
    qtbot.enaml_app = qt_app
    with close_all_windows(qtbot), close_all_popups(qtbot):
        yield qtbot



#------------------------------------------------------------------------------
# Copyright (c) 2019, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
#------------------------------------------------------------------------------
import sys
from textwrap import dedent

import pytest

from enaml.core.alias import Alias
from utils import compile_source


SUBSCRIPTION_BLOCK_TESTS = {
    # What should not work
    'leftshift newline in func': (SyntaxError, """
        enamldef Main(Window):
            Container:
                Label:
                    func test():
                        x = 1 <<
                            8
                        return str(x)
                    text = test()
        """),
    'leftshfit newline in attr': (SyntaxError, """
        enamldef Main(Window):
            Container:
                Label:
                    attr x = 1 <<
                                8
                    text = str(x)
        """),
    'leftshift newline in if': (SyntaxError, """
        enamldef Main(Window):
            Container:
                Label:
                    func test():
                        if 1 <<
                            8:
                            return 'this is bad'
                    text = str(test())
        """),
    'yield in sub block': (SyntaxError, """
        enamldef Main(Window):
            Container:
                Label:
                    text <<
                        for i in range(10):
                            yield i
        """),
    # What should still work
    'leftshift newline in parenthesis': (None, """
        enamldef Main(Window):
            Container:
                Label:
                    text = str(1 <<
                            8)
        """),
    'leftshift token in func': (None, """
        enamldef Main(Window):
            Container:
                Label:
                    func test():
                        x = 1 << 8
                        return str(x)
                    text = test()
        """),
    'leftshift token in attr': (None, """
        enamldef Main(Window):
            Container:
                Label:
                    attr x = 1 << 8
                    text = str(x)
        """),
    'leftshift newline update attr': (None, """
        enamldef Main(Window):
            Container:
                Label:
                    text <<
                        return str(1 << 8)
        """),
    'leftshift update attr': (None, """
        enamldef Main(Window):
            Container:
                Label:
                    text << str(1 << 8)
        """),
    'sub block var': (None, """
        enamldef Main(Window):
            Container:
                Label:
                    text <<
                        x = 10
                        if x:
                            return 'ten'
                        return 'not ten'
        """),
}


@pytest.mark.parametrize('name', SUBSCRIPTION_BLOCK_TESTS.keys())
def test_subscription_block_syntax(name):
    """Test a subscription block operator does not interfere with the leftshift
    operator.

    """
    error, code = SUBSCRIPTION_BLOCK_TESTS[name]
    source = dedent("""
    from enaml.widgets.api import *
    %s
    """ % code.lstrip())
    if error is not None:
        with pytest.raises(error):
            Main = compile_source(source, 'Main')
            window = Main()
    else:
        Main = compile_source(source, 'Main')
        window = Main()

# The following tests need to ensure that assignment in subscription block
# (either through a direct assignment or through the use of the walrus
# operator) do not cause issue with the tracer.
# Assignments can target the following scopes:
# - local to the block (only scope that the walrus operator target)
# - global module scope
# - widget and dynamic scope that can only be accessed qualified names (self.)

def test_subscription_block_observers1():
    """Test a subscription block operator.

    """
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Main(Window):
        alias label
        attr a = 1
        attr b = 2
        Label: label:
            text <<
                if a & 1:
                    return 'a'
                elif b & 1:
                    return 'b'
                return 'none'

    """)
    Main = compile_source(source, 'Main')
    window = Main()
    label = window.label
    assert label.text == 'a'
    window.a = 2
    assert label.text == 'none'  # 2 and 2
    window.b = 1
    assert label.text == 'b'
    window.a = 1
    assert label.text == 'a'


def test_subscription_block_observers2():
    """Test that intermediate assignments (local) do not confuse the tracer.

    """
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Main(Window):
        alias label
        attr a = 1
        attr b = 2
        Label: label:
            text <<
                b = a
                if b & 1:
                    return 'a'
                return 'none'

    """)
    Main = compile_source(source, 'Main')
    window = Main()
    label = window.label
    assert label.text == 'a'
    window.a = 2
    assert label.text == 'none'
    window.b = 1
    assert label.text == 'none'
    window.a = 1
    assert label.text == 'a'


def test_subscription_block_observers3():
    """Test that handling recursive behavior works.

    """
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Main(Window): main:
        alias label
        attr a = 1
        attr b = 2
        Label: label:
            text <<
                value = 'none'
                if a & 1:
                    value = 'a'
                elif b & 1:
                    value = 'b'
                main.b = 2
                return value

    """)
    Main = compile_source(source, 'Main')
    window = Main()
    label = window.label
    assert label.text == 'a'
    window.a = 2
    assert label.text == 'none'  # 2 and 2
    window.b = 1
    assert label.text == 'b'
    assert window.b == 2
    window.a = 1
    assert label.text == 'a'


def test_subscription_block_observers4():
    """Test that global variable shadowed by local can be accessed.

    """
    source = dedent("""\
    from enaml.widgets.api import *

    b = 2

    enamldef Main(Window): main:
        alias label
        attr a = 1
        attr b = 2
        attr counter = 0
        Label: label:
            text <<
                global b
                main.counter += 1
                if a & 1:
                    return 'a'
                elif b & 1:
                    return 'b'
                return 'none'

    """)
    Main = compile_source(source, 'Main')
    window = Main()
    label = window.label
    assert label.text == 'a'
    assert window.counter == 1
    window.a = 2
    assert label.text == 'none'  # 2 and 2
    assert window.counter == 2
    window.b = 1
    assert label.text == 'none'
    assert window.counter == 2
    window.a = 1
    assert label.text == 'a'
    assert window.counter == 3


def test_subscription_block_observers5():
    """Test that global variable shadowed by local do not block notification
    on qualified name access.

    """
    source = dedent("""\
    from enaml.widgets.api import *

    b = 2

    enamldef Main(Window): main:
        alias label
        attr a = 1
        attr b = 2
        attr counter = 0
        Label: label:
            text <<
                global b
                main.counter += 1
                if a & 1:
                    return 'a'
                elif main.b & 1:
                    return 'b'
                return 'none'

    """)
    Main = compile_source(source, 'Main')
    window = Main()
    label = window.label
    assert label.text == 'a'
    assert window.counter == 1
    window.a = 2
    assert label.text == 'none'  # 2 and 2
    assert window.counter == 2
    window.b = 1
    assert label.text == 'b'
    assert window.counter == 3
    window.a = 1
    assert label.text == 'a'
    assert window.counter == 4


@pytest.mark.skipif(sys.version_info < (3, 8), reason="Require Python 3.8+")
def test_subscription_operator_walrus():
    """Test handling the walrus operator inside a subscription.

    """
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Main(Window):
        alias label
        alias label2
        attr a = 1
        attr b = 2
        attr c = 5
        Label: label:
            text << f"{a + (c := b + 1)}"
        Label: label2:
            text << f"{c}"

    """)
    Main = compile_source(source, 'Main')
    window = Main()
    label = window.label
    assert label.text == '4'
    # This is the same behavior as everywhere else where access can use a bare name
    # but assignment needs a qualified name.
    assert window.label2.text == '5'


#------------------------------------------------------------------------------
# Copyright (c) 2020, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
#------------------------------------------------------------------------------
import gc

import pytest

from enaml.signaling import Signal, BoundSignal


class SignalTester:

    signal = Signal()

    def __init__(self):
        self.counter = 0
        self.last_args = None
        self.last_kwargs = None

    def slot(self, *args, **kwargs):
        self.counter += 1
        self.last_args = args
        self.last_kwargs = kwargs


def test_signal_bound_method():
    """Test creating, connecting, emitting and disconnecting a signal.

    """
    s = SignalTester()
    bs1 = s.signal
    bs2 = s.signal
    assert bs1 is not bs2
    assert isinstance(bs1, BoundSignal)

    s.signal.connect(s.slot)
    bs1.connect(s.slot)
    bs2.connect(s.slot)

    s.signal.emit(1, 2, 3)
    assert s.counter == 3
    assert s.last_args == (1, 2, 3)
    assert s.last_kwargs == {}

    s.signal.disconnect(s.slot)

    s.signal(4, 5, c=5, h=6)
    assert s.counter == 5
    assert s.last_args == (4, 5)
    assert s.last_kwargs == dict(c=5, h=6)

    s.signal.disconnect(s.slot)
    s.signal(a=1)
    assert s.counter == 6
    assert s.last_args == ()
    assert s.last_kwargs == dict(a=1)

    s.signal.disconnect(s.slot)
    s.signal()
    assert s.counter == 6


def test_signal_disconnect_all():
    """Test disconnecting all slots connected the bound signals.

    """
    c = 0
    last_args = ()
    last_kwargs = {}
    def dummy_slot(*args, **kwargs):
        nonlocal c, last_args, last_kwargs
        c += 1
        last_args = args
        last_kwargs = kwargs

    s = SignalTester()
    s.signal.connect(s.slot)
    s.signal(4, 5, c=5, h=6)
    assert s.counter == 1

    s.signal.connect(dummy_slot)
    s.signal.emit()
    assert s.counter == 2
    assert c == 1
    assert last_args == ()
    assert last_kwargs == dict()

    s.signal.emit(1, 2)
    assert s.counter == 3
    assert c == 2
    assert last_args == (1, 2)
    assert last_kwargs == dict()

    s.signal.emit(a=1)
    assert s.counter == 4
    assert c == 3
    assert last_args == ()
    assert last_kwargs == dict(a=1)

    s.signal.emit(1, a=1)
    assert s.counter == 5
    assert c == 4
    assert last_args == (1,)
    assert last_kwargs == dict(a=1)

    SignalTester.signal.disconnect_all(s)
    s.signal.emit()
    assert s.counter == 5
    assert c == 4


def test_signal_bad_creation():
    """Test handling bad arguments to Signal.

    """
    with pytest.raises(TypeError):
        Signal(1)

    with pytest.raises(TypeError):
        Signal(a=1)


def test_signal_set_del():
    """Test setting/deleting a signal

    """
    s = SignalTester()
    with pytest.raises(AttributeError):
        s.signal = 1

    s = SignalTester()
    s.signal.connect(s.slot)
    s.signal(4, 5, c=5, h=6)
    assert s.counter == 1

    del s.signal
    s.signal.emit()
    assert s.counter == 1


def test_bound_signal_comparison():
    """Test comparing different bound signals.

    """
    s = SignalTester()
    assert s.signal == s.signal


def test_manual_bound_signal_creation():
    """Test creating manually a BoundSignal.

    """
    s = SignalTester()
    with pytest.raises(TypeError):
        sb = BoundSignal(SignalTester.signal, s)

    import weakref
    sb = BoundSignal(SignalTester.signal, weakref.WeakMethod(s.slot))



#------------------------------------------------------------------------------
# Copyright (c) 2018, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
#------------------------------------------------------------------------------
import os
import sys
import ast
import enaml
import pytest
import traceback
from textwrap import dedent

from enaml.compat import PY39


def validate_ast(py_node, enaml_node, dump_ast=False, offset=0):
    """Validate each node of an ast against another ast.

    Typically used to compare an AST generated by the Python paser and one
    generated by the enaml parser.

    """
    if dump_ast:
        print('Python node:\n', ast.dump(py_node))
        print('Enaml node:\n', ast.dump(enaml_node))
    assert type(py_node) == type(enaml_node)
    if isinstance(py_node, ast.AST):
        for name, field in ast.iter_fields(py_node):
            if name == 'ctx':
                assert type(field) == type(getattr(enaml_node, name))
            elif name not in ('lineno', 'col_offset'):
                field2 = getattr(enaml_node, name, None)
                print('    '*offset, 'Validating:', name)
                validate_ast(field, field2, offset=offset+1)
    elif isinstance(py_node, list):
        if len(py_node) != len(enaml_node):
            return False
        for i, n1 in enumerate(py_node):
            print('    '*offset, 'Validating', i+1, 'th element')
            validate_ast(n1, enaml_node[i], offset=offset+1)
    else:
        assert py_node == enaml_node


def test_syntax_error_traceback_correct_path(tmpdir):
    """ Test that a syntax error retains the path to the file

    """
    test_module_path = os.path.join(tmpdir.strpath, 'view.enaml')

    with open(os.path.join(tmpdir.strpath, 'test_main.enaml'), 'w') as f:
        f.write(dedent("""
        from enaml.widgets.api import Window, Container, Label
        from view import CustomView

        enamldef MyWindow(Window): main:
            CustomView:
                pass

        """))

    with open(test_module_path, 'w') as f:
        f.write(dedent("""
        from enaml.widgets.api import Container, Label

        enamldef CustomLabel(Container):
            Label # : missing intentionally
                text = "Hello world"

        """))

    try:
        sys.path.append(tmpdir.strpath)
        with enaml.imports():
            from test_main import MyWindow
        assert False, "Should raise a syntax error"
    except Exception as e:
        tb = traceback.format_exc()
        print(tb)
        lines = tb.strip().split("\n")
        assert ('File "{}", line 5'.format(test_module_path) in
            (lines[-3] if PY39 else lines[-4]))
    finally:
        sys.path.remove(tmpdir.strpath)



#------------------------------------------------------------------------------
# Copyright (c) 2020, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
#------------------------------------------------------------------------------
import sys
import ast
from textwrap import dedent

import pytest

from enaml.core.parser import parse

from .test_parser import validate_ast

TEST_SOURCE = {
    ':= in expr': r"""
    a = (b := 1)
    """,
    ':= in tuple': r"""
    a = (2, b := 1)
    """,
    ':= in list': r"""
    a = [2, b := 1]
    """,
    ':= in if': r"""
    if b := 1:
        pass
    """,
    ':= in elif': r"""
    a = 0
    if a:
        pass
    elif b := 1:
        pass
    """,
    ':= in while': r"""
    while b := 1:
        pass
    """,
    ':= in function argument': r"""
    f(a, b := c or None)
    """
}


@pytest.mark.skipif(sys.version_info < (3, 8), reason='Requires Python 3.8')
@pytest.mark.parametrize('desc', TEST_SOURCE.keys())
def test_walrus_operator(desc):
    """Test that we produce valid ast use of the := walrus operator.

    """
    src = dedent(TEST_SOURCE[desc]).strip()
    print(src)
    # Ensure it's valid
    py_ast = ast.parse(src)
    enaml_ast = parse(src).body[0].ast
    validate_ast(py_ast.body[0], enaml_ast.body[0], True)


TEST_BAD_SOURCE = {
    ':= in while': r"""
    while 2 := 1:
        pass
    """,
    ':= in function argument': r"""
    f(a, 2 := c or None)
    """
}

@pytest.mark.skipif(sys.version_info < (3, 8), reason='Requires Python 3.8')
@pytest.mark.parametrize('desc', TEST_BAD_SOURCE.keys())
def test_invalid_walrus_operator(desc):
    """Test that we produce valid ast use of the := walrus operator.

    """
    src = dedent(TEST_BAD_SOURCE[desc]).strip()
    print(src)
    with pytest.raises(SyntaxError):
        enaml_ast = parse(src).body[0].ast



#------------------------------------------------------------------------------
# Copyright (c) 2013-2017, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
#------------------------------------------------------------------------------
import os
import sys
import enaml
import struct
import marshal
import zipfile
from importlib.machinery import ModuleSpec

import pytest

from enaml.core.parser import parse
from enaml.core.enaml_compiler import EnamlCompiler
from enaml.core.import_hooks import MAGIC_NUMBER, make_file_info, EnamlImporter
from utils import wait_for_window_displayed, is_qt_available


pytestmark = pytest.mark.skipif(not is_qt_available(),
                                reason='Requires a Qt binding')


def generate_cache(path):
    #: Read
    with open(path, 'r') as f:
        enaml_code = f.read()

    #: Compile
    ast = parse(enaml_code, filename=path)
    code = EnamlCompiler.compile(ast, path)

    #: Generate cache
    with open('tmp.enamlc', 'wb') as f:
        f.write(MAGIC_NUMBER)
        f.write(struct.pack('i', int(os.path.getmtime(path))))
        marshal.dump(code, f)
    with open('tmp.enamlc', 'rb') as f:
        data = f.read()
    #: Cleanup
    if os.path.exists('tmp.enamlc'):
        os.remove('tmp.enamlc')

    return data


def make_library(lib):
    # Create a library.zip with the examples.

    root = os.path.normpath(os.path.join(os.path.dirname(__file__), '..', '..',
                            'examples', 'widgets'))

    with zipfile.ZipFile(lib, 'w') as zf:
        for src, dst in [
                         (os.path.join(root, 'buttons.enaml'),
                          'buttons.enaml'),
                         (None, 'package/__init__.py'),
                         (os.path.join(root, 'form.enaml'),
                          'package/form.enaml'),
                         (None, 'package/subpackage/__init__.py'),
                         (os.path.join(root, 'slider.enaml'),
                          'package/subpackage/slider.enaml'),
                         ]:
            if src:
                zf.write(src, dst, compress_type=zipfile.ZIP_DEFLATED)
            else:
                #: Create the __init__.py
                zf.writestr(dst, '', compress_type=zipfile.ZIP_DEFLATED)

        #: Generate cache for splitter and notebook
        for src, dst in [
                (os.path.join(root, 'splitter.enaml'),
                 make_file_info('splitter.enaml').cache_path),
                (os.path.join(root, 'notebook.enaml'),
                 make_file_info('package/subpackage/'
                                'notebook.enaml').cache_path)
                ]:
            zf.writestr(dst, generate_cache(src),
                        compress_type=zipfile.ZIP_DEFLATED)


@pytest.fixture
def zip_library():
    """ Add and remove library.zip from the sys path

    """
    lib = 'library.zip'
    # Create the zip
    make_library(lib)

    # Add to sys.path
    sys.path.append(lib)

    # Pytest handle the cleanup without the need for try finally close
    yield lib

    sys.path.remove(lib)
    #: Cleanup
    if os.path.exists(lib):
        os.remove(lib)


def assert_window_displays(enaml_qtbot, enaml_sleep, window):
    #: Make sure the imported library actually works
    window.show()
    window.send_to_front()
    wait_for_window_displayed(enaml_qtbot, window)
    enaml_qtbot.wait(enaml_sleep*1000)


def test_zipimport_from_module(enaml_qtbot, enaml_sleep, zip_library):
    with enaml.imports():
        # Test import from library.zip/buttons.enaml
        import buttons

    assert buttons.__name__ == "buttons"
    assert buttons.__file__
    assert buttons.__cached__
    assert isinstance(buttons.__loader__, EnamlImporter)
    assert isinstance(buttons.__spec__, ModuleSpec)

    assert_window_displays(enaml_qtbot, enaml_sleep, buttons.Main())


def test_zipimport_from_package(enaml_qtbot, enaml_sleep, zip_library):
    with enaml.imports():
        # Test import from library.zip/package/form.enaml
        from package import form

    assert_window_displays(enaml_qtbot, enaml_sleep, form.Main())


def test_zipimport_from_subpackage(enaml_qtbot, enaml_sleep, zip_library):
    with enaml.imports():
        # Test import from library.zip/package/subpackage/slider.enaml
        from package.subpackage import slider

    assert_window_displays(enaml_qtbot, enaml_sleep, slider.Main())


def test_zipimport_module_from_cache(enaml_qtbot, enaml_sleep, zip_library):
    with enaml.imports():
        # Test import from
        # library.zip/__enamlcache__/slider-enaml-py<ver>-cv<ver>.enamlc
        import splitter

    assert_window_displays(enaml_qtbot, enaml_sleep, splitter.Main())


def test_zipimport_package_from_cache(enaml_qtbot, enaml_sleep, zip_library):
    with enaml.imports():
        # Test import from enamlcache subpackage within zip
        # Test import from library.zip/package/subpackage/__enamlcache__/
        #                                 notebook-enaml-py<ver>-cv<ver>.enamlc
        from package.subpackage import notebook
    assert_window_displays(enaml_qtbot, enaml_sleep, notebook.Main())



#------------------------------------------------------------------------------
# Copyright (c) 2013-2018, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
#------------------------------------------------------------------------------
""" Generate the Example Documentation for the Enaml Examples

Run as part of the documentation build script. Requires PyQt4.
Look for example enaml files with the line:

<< autodoc-me >>

Generate an rst file, then open the app and take a snapshot.

Note: Many examples have dependencies outside of the normal Enaml dependencies.
These will need to be installed for the snapshots to be generated.

"""
from __future__ import print_function
import argparse
import os
import re
import shutil
import sys
from textwrap import dedent

from atom.api import Atom, Str, Value
import enaml
from enaml.qt.qt_application import QtApplication
from enaml.qt.QtWidgets import QApplication
from enaml.qt.QtTest import QTest


class SnapShot(Atom):
    """ Generate a snapshot of an enaml view.

    """

    #: The snapshot save path.
    path = Str()

    #: The enaml view object.
    view = Value()

    def _observe_view(self, change):
        """ Move window and allow it to draw before taking the snapshot.

        """
        if change['type'] == 'create':
            self.view.initial_position = (10, 10)
            self.view.always_on_top = True

    def snapshot(self):
        """ Take a snapshot of the window and close it.

        """
        widget = self.view.proxy.widget
        framesize = widget.window().frameSize()
        screen = QApplication.primaryScreen()
        screen.grabWindow(
            QApplication.desktop().winId(),
            widget.x(), widget.y(),
            framesize.width(), framesize.height()
            ).save(self.path)
        self.view.close()


def save_snapshot_of_module(mod, image_path):
    view = mod.Main()
    snapshot = SnapShot(path=image_path, view=view)
    view.show()
    QTest.qWaitForWindowExposed(view.proxy.widget)
    QTest.qWait(500)
    snapshot.snapshot()
    QTest.qWait(100)


def extract_docstring(script_text):
    """ Extract the docstring from an example Enaml script. """

    # The docstring is found between the first two '"""' strings.
    return script_text.split('"""')[1]


def clean_docstring(docstring):
    """ Convert a docstring into ReStructuredText format. """

    docstring = docstring.replace('<< autodoc-me >>\n', '').strip()
    # Find backquoted identifiers, and double the backquotes to match RST.
    docstring = re.sub(
        pattern=r"`(?P<identifier>[A-Za-z_]+)`",  # Backquoted identifiers
        repl=r"``\g<identifier>``",  # Double backquoted identifiers
        string=docstring)

    return docstring


SCREENSHOT_RST_TEMPLATE = dedent("""
    Screenshot
    -------------------------------------------------------------------------------

    .. image:: images/{name}
    """)

EXAMPLE_DOC_RST_TEMPLATE = dedent("""
    ..
      NOTE: This RST file was generated by `make examples`.
      Do not edit it directly.
      See docs/source/examples/example_doc_generator.py

    {title} Example
    ===============================================================================

    {docstring_rst}

    .. TIP:: To see this example in action, download it from
     :download:`{name} <../../../{path}>`
     and run::

       $ enaml-run {name}.enaml

    {screenshot_rst}
    Example Enaml Code
    -------------------------------------------------------------------------------
    .. literalinclude:: ../../../{path}
        :language: enaml
    """)


def generate_example_doc(docs_path, script_path):
    """ Generate an RST and a PNG for an example file.

    Parameters
    ----------
    docs_path : str
         Full path to enaml/docs/source/examples
    script_path : str
         Full path to the example enaml file
    """
    script_name = os.path.basename(script_path)
    script_name = script_name[:script_name.find('.')]
    print('generating doc for %s' % script_name)

    script_title = script_name.replace('_', ' ').title()
    script_image_name = 'ex_' + script_name + '.png'
    image_path = os.path.join(docs_path, 'images', script_image_name)
    rst_path = os.path.join(
        docs_path, 'ex_' + script_name + '.rst')
    relative_script_path = script_path[
        script_path.find('examples'):].replace('\\', '/')

    # Add the script to the Python Path
    old_python_path = sys.path
    sys.path = sys.path + [os.path.dirname(script_path)]

    snapshot_success = False
    with enaml.imports():
        try:
            mod = __import__(script_name)
            save_snapshot_of_module(mod, image_path)
            snapshot_success = True
        except Exception as err:
            print('Could not snapshot: %s' % script_name)
            print('    %s' % err)
        finally:
            # The import leaves behind a cache. Clean it up.
            enaml_cache_dir = os.path.join(
                os.path.dirname(script_path), '__enamlcache__')
            shutil.rmtree(enaml_cache_dir)

    # Restore Python path.
    sys.path = old_python_path

    with open(os.path.join(script_path)) as fid:
        script_text = fid.read()

    docstring = clean_docstring(extract_docstring(script_text))

    screenshot_rst = '' if not snapshot_success else (
            SCREENSHOT_RST_TEMPLATE.format(name=script_image_name))

    example_doc_rst = EXAMPLE_DOC_RST_TEMPLATE.format(
        title=script_title,
        name=script_name,
        path=relative_script_path,
        docstring_rst=docstring,
        screenshot_rst=screenshot_rst)

    with open(rst_path, 'wb') as rst_output_file:
        rst_output_file.write(example_doc_rst.lstrip().encode())


def main(white_list):
    """ Generate documentation for all enaml examples requesting autodoc.

    Looks in enaml/examples for all enaml files, then looks for the line:
    << auto-doc >>

    If the line appears in the script, generate an RST and PNG for the example.
    """
    app = QtApplication()
    docs_path = os.path.dirname(__file__)
    base_path = '../../../examples'
    base_path = os.path.realpath(os.path.join(docs_path, base_path))

    # Find all the files in the examples directory with a .enaml extension
    # that contain the pragma '<< autodoc-me >>', and generate .rst files for
    # them.
    for dirname, dirnames, filenames in os.walk(base_path):
        files = [os.path.join(dirname, f)
                 for f in filenames if f.endswith('.enaml')]
        if white_list:
            files = [f for f in files
                     if any([f.endswith(w) for w in white_list])]
        for fname in files:
            with open(fname, 'rb') as fid:
                data = fid.read()
            if b'<< autodoc-me >>' in data.splitlines():
                    generate_example_doc(docs_path, fname)


if __name__ == '__main__':

    doc = ("Enaml example documentation generator.\n\n"
           "By default all examples with the << auto-doc >> directive are "
           "processed.")
    parser = argparse.ArgumentParser(description=doc)
    parser.add_argument('-f', '--filenames',
                        help='Files for which to generate the docs',
                        action='store', nargs='+')

    args = parser.parse_args()

    main(args.filenames)



#------------------------------------------------------------------------------
# Copyright (c) 2013-2018, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
#------------------------------------------------------------------------------
from textwrap import dedent

import pytest

from enaml.core.alias import Alias
from utils import compile_source


def test_alias_attributes():
    """Test accessing an alias attributes.

    """
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Main(Window):
        alias fd
        alias fd_txt: fd.text

        Field: fd:
            name = 'content'

    """)
    win = compile_source(source, 'Main')
    for alias_name, target, chain, canset in [('fd', 'fd', (), False),
                                              ('fd_txt', 'fd', ('text',), True)
                                              ]:
        alias = getattr(win, alias_name)
        assert alias.target == target
        assert alias.chain == chain
        assert alias.key
        assert alias.canset is canset

    with pytest.raises(TypeError):
        alias.canset = 1


def test_alias_resolve():
    """Test resolving an alias.

    """
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Main(Window):

        alias ct
        Container: ct:
            alias fd
            Field: fd:
                name = 'content'

    """)
    win = compile_source(source, 'Main')()
    key = list(win._d_storage.keys())[0]
    alias = Alias('ct', (), key)
    assert alias.resolve(win) == (win.ct, None)

    alias = Alias('ct', ('fd',), key)
    assert alias.resolve(win) == (win.ct.fd, None)

    alias = Alias('ct', ('fd', 'text'), key)
    assert alias.resolve(win) == ('', None)

    alias.canset = True
    assert alias.resolve(win) == (win.ct.fd, 'text')

    alias = Alias('unknown', (), key)
    with pytest.raises(RuntimeError):
        alias.resolve(win)

    alias = Alias('ct', ('fd', 'unknown'), key)
    with pytest.raises(AttributeError):
        alias.resolve(win)


def test_simple_alias_get_error():
    """Test handling an error getting a simple alias.

    """
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Main(Window):

        alias ct
        Container: ct:
            alias fd
            Field: fd:
                name = 'content'

    """)
    win_cls = compile_source(source, 'Main')
    key = list(win_cls()._d_storage.keys())[0]
    win_cls.alias = Alias('unknown', (), key)
    win = win_cls()

    with pytest.raises(RuntimeError) as excinfo:
        win.alias
    assert 'failed to load alias' in excinfo.exconly()


def test_chained_alias_get_error():
    """Test handling an error getting a chained alias.

    """
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Main(Window):

        alias ct
        Container: ct:
            alias fd
            Field: fd:
                name = 'content'

    """)
    win_cls = compile_source(source, 'Main')
    key = list(win_cls()._d_storage.keys())[0]
    win_cls.alias = Alias('ct', ('unknown',), key)
    win_cls.alias2 = Alias('unknown', ('unknown',), key)
    win = win_cls()

    with pytest.raises(AttributeError):
        win.alias
    with pytest.raises(RuntimeError):
        win.alias2


def test_alias_set_canset_false():
    """Test handling an error setting a simple alias.

    """
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Main(Window):

        alias ct
        Container: ct:
            alias fd
            Field: fd:
                name = 'content'

    """)
    win_cls = compile_source(source, 'Main')
    key = list(win_cls()._d_storage.keys())[0]
    win_cls.alias = Alias('unknown', (), key)
    win = win_cls()

    with pytest.raises(AttributeError):
        win.alias = 1


def test_simple_alias_set_error():
    """Test handling an error setting a simple alias.

    """
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Main(Window):

        alias ct
        Container: ct:
            alias fd
            Field: fd:
                name = 'content'

    """)
    win_cls = compile_source(source, 'Main')
    key = list(win_cls()._d_storage.keys())[0]
    win_cls.alias = Alias('unknown', (), key)
    win_cls.alias.canset = True
    win = win_cls()

    with pytest.raises(RuntimeError) as excinfo:
        win.alias = 1
    assert 'failed to load alias' in excinfo.exconly()


def test_chained_alias_set_error():
    """Test handling an error setting a chained alias.

    """
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Main(Window):

        alias ct
        Container: ct:
            alias fd
            Field: fd:
                name = 'content'

    """)
    win_cls = compile_source(source, 'Main')
    key = list(win_cls()._d_storage.keys())[0]
    win_cls.alias = Alias('ct', ('fd', 'unknown'), key)
    win_cls.alias.canset = True
    win_cls.alias2 = Alias('unknown', ('unknown',), key)
    win_cls.alias2.canset = True
    win = win_cls()

    with pytest.raises(AttributeError):
        win.alias = 1
    with pytest.raises(RuntimeError):
        win.alias2 = 1


#------------------------------------------------------------------------------
# Alias Syntax
#------------------------------------------------------------------------------
def test_syntax_1():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias pb
        PushButton: pb:
            name = 'button'

    enamldef Main(Window):
        Content:
            name = 'content'

    """)
    compile_source(source, 'Main')


def test_syntax_2():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias pc: pb
        PushButton: pb:
            name = 'button'

    enamldef Main(Window):
        Content:
            name = 'content'

    """)
    compile_source(source, 'Main')


def test_syntax_3():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias pc: pb.text
        PushButton: pb:
            name = 'button'

    enamldef Main(Window):
        Content:
            name = 'content'

    """)
    compile_source(source, 'Main')


#------------------------------------------------------------------------------
# Bad Alias Syntax
#------------------------------------------------------------------------------
def test_bad_syntax_1():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias pb.text
        PushButton: pb:
            text = 'spam'

    enamldef Main(Window):
        Content:
            name = 'content'

    """)
    with pytest.raises(SyntaxError):
        compile_source(source, 'Main')


def test_bad_syntax_2():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias pb text
        PushButton: pb:
            text = 'spam'

    enamldef Main(Window):
        Content:
            name = 'content'

    """)
    with pytest.raises(SyntaxError):
        compile_source(source, 'Main')


def test_bad_syntax_3():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias pb: pb text
        PushButton: pb:
            text = 'spam'

    enamldef Main(Window):
        Content:
            name = 'content'

    """)
    with pytest.raises(SyntaxError):
        compile_source(source, 'Main')


#------------------------------------------------------------------------------
# Alias References
#------------------------------------------------------------------------------
def test_ref_1():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias pb
        PushButton: pb:
            name = 'button'

    enamldef Main(Window):
        Content:
            name = 'content'

    """)
    main = compile_source(source, 'Main')()
    button = main.find('button')
    content = main.find('content')
    assert content.pb is button


def test_ref_2():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias pb: pb
        PushButton: pb:
            name = 'button'

    enamldef Main(Window):
        Content:
            name = 'content'

    """)
    main = compile_source(source, 'Main')()
    button = main.find('button')
    content = main.find('content')
    assert content.pb is button


def test_ref_3():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias foo: pb
        PushButton: pb:
            name = 'button'

    enamldef Main(Window):
        Content:
            name = 'content'

    """)
    main = compile_source(source, 'Main')()
    button = main.find('button')
    content = main.find('content')
    assert content.foo is button


def test_ref_4():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias pb: pb.text
        PushButton: pb:
            text = 'spam'

    enamldef Main(Window):
        Content:
            name = 'content'

    """)
    main = compile_source(source, 'Main')()
    content = main.find('content')
    assert content.pb == 'spam'


def test_ref_5():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Other(Container):
        alias slider
        Slider: slider:
            name = 'slider'

    enamldef Content(Container):
        alias other
        Other: other:
            pass

    enamldef Main(Window):
        Content:
            name = 'content'

    """)
    main = compile_source(source, 'Main')()
    slider = main.find('slider')
    content = main.find('content')
    assert content.other.slider is slider


def test_ref_6():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Other(Container):
        alias value: slider.value
        Slider: slider:
            value = 50

    enamldef Content(Container):
        alias other
        Other: other:
            pass

    enamldef Main(Window):
        Content:
            name = 'content'

    """)
    main = compile_source(source, 'Main')()
    content = main.find('content')
    assert content.other.value == 50


def test_ref_7():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Other(Container):
        alias value: slider.value
        Slider: slider:
            value = 50

    enamldef Content(Container):
        alias value: other.value
        Other: other:
            pass

    enamldef Main(Window):
        Content:
            name = 'content'

    """)
    main = compile_source(source, 'Main')()
    content = main.find('content')
    assert content.value == 50


def test_ref_8():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Other(Container):
        alias slider
        Slider: slider:
            value = 50

    enamldef Content(Container):
        alias value: other.slider.value
        Other: other:
            pass

    enamldef Main(Window):
        Content:
            name = 'content'

    """)
    main = compile_source(source, 'Main')()
    content = main.find('content')
    assert content.value == 50


#------------------------------------------------------------------------------
# Bad Alias Reference
#------------------------------------------------------------------------------
def test_bad_ref_1():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias pc
        PushButton: pb:
            text = 'spam'

    enamldef Main(Window):
        Content:
            name = 'content'

    """)
    with pytest.raises(TypeError):
        compile_source(source, 'Main')


def test_bad_ref_2():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias pc: pd
        PushButton: pb:
            text = 'spam'

    enamldef Main(Window):
        Content:
            name = 'content'

    """)
    with pytest.raises(TypeError):
        compile_source(source, 'Main')


def test_bad_ref_3():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias pc: pb.tex
        PushButton: pb:
            text = 'spam'

    enamldef Main(Window):
        Content:
            name = 'content'

    """)
    with pytest.raises(TypeError):
        compile_source(source, 'Main')


def test_bad_ref_4():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias pc: pb.text.spam
        PushButton: pb:
            text = 'spam'

    enamldef Main(Window):
        Content:
            name = 'content'

    """)
    with pytest.raises(TypeError):
        compile_source(source, 'Main')


def test_bar_ref_5():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Other(Container):
        alias value: slider.value
        Slider: slider:
            value = 50

    enamldef Content(Container):
        alias value: other.valued
        Other: other:
            pass

    enamldef Main(Window):
        Content:
            name = 'content'

    """)
    with pytest.raises(TypeError):
        compile_source(source, 'Main')


def test_bad_ref_6():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Other(Container):
        alias slider
        Slider: slider:
            value = 50

    enamldef Content(Container):
        alias value: other.slider.valsue
        Other: other:
            pass

    enamldef Main(Window):
        Content:
            name = 'content'

    """)
    with pytest.raises(TypeError):
        compile_source(source, 'Main')


#------------------------------------------------------------------------------
# Alias Binding
#------------------------------------------------------------------------------
def test_bind_1():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias pb: pb.text
        PushButton: pb:
            name = 'button'

    enamldef Main(Window):
        Content:
            pb = 'foo'

    """)
    main = compile_source(source, 'Main')()
    button = main.find('button')
    assert button.text == 'foo'


def test_bind_2():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Other(Container):
        alias slider
        Slider: slider:
            name = 'slider'

    enamldef Content(Container):
        alias value: other.slider.value
        Other: other:
            pass

    enamldef Main(Window):
        Content:
            value = 50

    """)
    main = compile_source(source, 'Main')()
    slider = main.find('slider')
    assert slider.value == 50


def test_bind_3():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Other(Container):
        alias value: slider.value
        Slider: slider:
            name = 'slider'
            value = 50

    enamldef Content(Container):
        alias value: other.value
        Other: other:
            pass

    enamldef Main(Window):
        Content:
            value = 42

    """)
    main = compile_source(source, 'Main')()
    slider = main.find('slider')
    assert slider.value == 42


#------------------------------------------------------------------------------
# Bad Alias Binding
#------------------------------------------------------------------------------
def test_bad_bind_1():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias pb
        PushButton: pb:
            name = 'button'

    enamldef Main(Window):
        Content:
            pb = 'foo'

    """)
    with pytest.raises(TypeError):
        compile_source(source, 'Main')


def test_bad_bind_2():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias text: pb.text
        PushButton: pb:
            name = 'button'

    enamldef Main(Window):
        Content:
            txt = 'foo'

    """)
    with pytest.raises(TypeError):
        compile_source(source, 'Main')


#------------------------------------------------------------------------------
# Extended Alias Binding
#------------------------------------------------------------------------------
def test_ex_bind_1():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias pb
        PushButton: pb:
            name = 'button'

    enamldef Main(Window):
        Content:
            pb.text = 'foo'

    """)
    main = compile_source(source, 'Main')()
    button = main.find('button')
    assert button.text == 'foo'


def test_ex_bind_2():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Other(Container):
        alias slider
        Slider: slider:
            name = 'slider'

    enamldef Content(Container):
        alias value: other.slider
        Other: other:
            pass

    enamldef Main(Window):
        Content:
            value.value = 50

    """)
    main = compile_source(source, 'Main')()
    slider = main.find('slider')
    assert slider.value == 50


def test_ex_bind_3():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Other(Container):
        alias slider
        Slider: slider:
            name = 'slider'

    enamldef Content(Container):
        alias value: other
        Other: other:
            pass

    enamldef Main(Window):
        Content:
            value.slider.value = 42

    """)
    main = compile_source(source, 'Main')()
    slider = main.find('slider')
    assert slider.value == 42


#------------------------------------------------------------------------------
# Bad Alias Binding
#------------------------------------------------------------------------------
def test_bad_ex_bind_1():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias pb
        PushButton: pb:
            name = 'button'

    enamldef Main(Window):
        Content:
            pbd.text = 'foo'

    """)
    with pytest.raises(TypeError):
        compile_source(source, 'Main')


def test_bad_ex_bind_2():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias pb
        PushButton: pb:
            name = 'button'

    enamldef Main(Window):
        Content:
            pb.txt = 'foo'

    """)
    with pytest.raises(TypeError):
        compile_source(source, 'Main')


def test_bad_ex_bind_3():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Other(Container):
        alias slider
        Slider: slider:
            name = 'slider'

    enamldef Content(Container):
        alias value: other.slider
        Other: other:
            pass

    enamldef Main(Window):
        Content:
            value.val = 50

    """)
    with pytest.raises(TypeError):
        compile_source(source, 'Main')


#------------------------------------------------------------------------------
# Alias Ordering
#------------------------------------------------------------------------------
def test_ordering():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        Field: field:
            alias this_text: field.text
        alias text: field.this_text

    enamldef Main(Window):
        Content:
            pass

    """)
    compile_source(source, 'Main')


#------------------------------------------------------------------------------
# Bad Alias Ordering
#------------------------------------------------------------------------------
def test_bar_ordering():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias text: field.this_text
        Field: field:
            alias this_text: field.text

    enamldef Main(Window):
        Content:
            pass

    """)
    with pytest.raises(TypeError):
        compile_source(source, 'Main')


#------------------------------------------------------------------------------
# Bad Alias Override
#------------------------------------------------------------------------------
def test_bad_override_1():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias background
        Field: background:
            pass

    enamldef Main(Window):
        Content:
            pass

    """)
    with pytest.raises(TypeError):
        compile_source(source, 'Main')


def test_bad_override_2():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias foo
        Field: foo:
            pass

    enamldef Content2(Content):
        alias foo: bar
        Field: bar:
            pass

    enamldef Main(Window):
        Content2:
            pass

    """)
    with pytest.raises(TypeError):
        compile_source(source, 'Main')


def test_bad_override_3():
    source = dedent("""\
    from enaml.widgets.api import *

    enamldef Content(Container):
        alias foo
        Field: foo:
            pass

    enamldef Content2(Content):
        attr foo

    enamldef Main(Window):
        Content2:
            pass

    """)
    with pytest.raises(TypeError):
        compile_source(source, 'Main')



#------------------------------------------------------------------------------
# Copyright (c) 2019, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
#-----------------------------------------------------------------------------
import sys
import ast
from textwrap import dedent

import pytest
from enaml.core.parser import parse

from .test_parser import validate_ast


def test_raise_exception():
    """Test raising an exception.

    """
    src = """raise Exception()"""
    py_ast = ast.parse(src).body[0]
    enaml_ast = parse(src).body[0].ast.body[0]
    validate_ast(py_ast, enaml_ast, True)


def test_reraise_exception():
    """Test raising an exception.

    """
    src = dedent("""
    try:
        a = ""/0
    except:
        raise
    """)
    py_ast = ast.parse(src).body[0]
    enaml_ast = parse(src).body[0].ast.body[0]
    validate_ast(py_ast, enaml_ast, True)


@pytest.mark.skipif(sys.version_info < (3,), reason='Requires Python 3')
def test_raise_exception_with_cause():
    """Test that we parse properly functions with a return type annotation.

    """
    src = dedent("""
    try:
        a = ""/0
    except Exception as e:
        raise Exception() from e
    """)
    py_ast = ast.parse(src).body[0]
    enaml_ast = parse(src).body[0].ast.body[0]
    validate_ast(py_ast, enaml_ast, True)



import os
import sys
import pytest
from utils import enaml_run

from enaml.application import Application, deferred_call
from enaml.runner import main


@pytest.fixture
def sys_argv():
    """ Fixture that saves sys.argv and restores it after the test completes

    """
    argv = sys.argv
    try:
        yield
    finally:
        sys.argv = argv


def test_runner(enaml_run, sys_argv):
    """Test invoking the runner application.

    """
    dir_path = os.path.abspath(os.path.split(os.path.dirname(__file__))[0])
    sys.argv = ['enaml-run',
                os.path.join(dir_path,
                             'examples', 'stdlib', 'mapped_view.enaml')]
    main()
